#
# Menu system for 20 character by 4 line screen
#   No editable section in here... Look in Settings.pm
#

package Menu;

use Exporter;
use Display;
use Settings;
use POSIX;
use IO;

@ISA	= qw ( Exporter );
@EXPORT	= qw ( menu_title menu_render menu_navigate menu_text_entry
               menu_select menu_yes_no menu_info menu_mini fit
               menu_until_keypress menu_scroll );

###########

my ( $display_type, $serial_port,
     $up_key, $down_key, $forward_key, $back_key,
     $setvol_cmd, $ln_scr_mode, $radio_pass_thru,
     $halt_cmd ) = settings_get ( );

if ( $display_type eq "vfd" ) {
  $pointer_char  = "29";
  $position_char = "31";
  $back_char     = "30";
  $select_char   = "25";
  $ln_scr_prc    = 50;

} elsif ($display_type eq "lcd" ) {
  $pointer_char  = "126";
  $position_char = "94";
  $back_char     = "127";
  $select_char   = "42";
  $ln_scr_prc    = 50;

} else {
  $pointer_char  = "62";
  $position_char = "94";
  $back_char     = "60";
  $select_char   = "42";
  $ln_scr_prc    = 80;

}

$blank = sprintf "%20c", 32;

#
# menu_title ( $title );
#   displays the menu title
#

sub menu_title  {

  my ( $title ) = @_;

  display_position ( 1, 1 );
  display_write ( fit ( $title, 14 ) );

}

#
# $result = fit ( $text, $length );
#   returns $text truncated or padded to $length characters
#

sub fit  {

  my ( $string, $length ) = @_;

  $cmd1 = "%-$length" . "s";
  $cmd2 = "%.$length" . "s";

  $string = sprintf "$cmd1", $string;
  $string = sprintf "$cmd2", $string;

  return $string;

}

#
# $result = reverse_fit ( $text, $length );
#   returns $text truncated or padded to $length characters, right justified
#

sub reverse_fit  {

  my ( $string, $length ) = @_;

  $cmd1 = "%+$length" . "s";
  $cmd2 = "%.$length" . "s";

  $string = sprintf "$cmd1", $string;
  $string = sprintf "$cmd2", $string;

  return $string;

}
#
# $result = center_fit ( $text, $length );
#

sub center_fit  {

  my ( $string, $length ) = @_;

  my $half   = int ( $length / 2 );
  my @ary    = split //, $string;
  my $size   = @ary;
  my $blanks = int ( $half - ( $size / 2 ) );

  if ( $blanks > 0 )  {
    $cmd = "%$blanks" . "c%s";
    $string = sprintf $cmd, 32, $string;

  }

  $cmd1 = "%-$length" . "s";
  $cmd2 = "%.$length" . "s";

  $string = sprintf "$cmd1", $string;
  $string = sprintf "$cmd2", $string;

  return $string;

}

#
# menu_render ( $selected_item, $back_option, @list_of_menu_items );
#   takes an array of menu items and displays the menu centered around
#   the $selected_item. $back_option is a boolean effecting the nav
#   character.
#

sub menu_render  {

  my ( @list ) = @_;

  my ( $selected, $back, $length, $x ) = 0;

  $selected = pop ( @list );
  $back = pop ( @list );
  $scr_line = pop ( @list );
  $length = @list;
  $length --;

  while ( $x <= $length )  {
    @list [ $x ] = fit ( @list [ $x ], 19 ) if ( $x ne $selected );
    $x ++;

  }

  if ( $length == -1 )  {
    print "menu list doesn't have any elements\n";

  } elsif ( $length == 0 )  {
    $return_line=2;
    $line1 = sprintf "%c%-19s", $pointer_char, display_scroll ( @$scr_line );
    $line2 = $blank;
    $line3 = $blank;
    if ( $back == 1 )  {
      $cmd = $set12;

    } else  {
      $cmd = $set8;

    }

  } elsif ( $length == 1 )  {
    if ( $selected == 0 )  {
      $return_line=2;
      $line1 = sprintf "%c%-19s", $pointer_char, display_scroll ( @$scr_line );
      $line2 = sprintf "%c%s", 32, fit ( @list [ 1 ], 19 );
      $line3 = $blank;
      if ( $back == 1 )  {
        $cmd = $set14;

      } else  {
        $cmd = $set10;

      }

    } else  {
      $return_line=3;
      $line1 = sprintf "%c%s", 32, fit ( @list [ 0 ], 19 );
      $line2 = sprintf "%c%-19s", $pointer_char, display_scroll ( @$scr_line );
      $line3 = $blank;
      if ( $back == 1 )  {
        $cmd = $set13;

      } else  {
        $cmd = $set9;

      }

    }

  } elsif ( $selected == 0 )  {
    $return_line=2;
    $line1 = sprintf "%c%-19s", $pointer_char, display_scroll ( @$scr_line );
    $line2 = sprintf "%c%s", 32, fit ( @list [ 1 ], 19 );
    $line3 = sprintf "%c%s", 32, fit ( @list [ 2 ], 19 );
    if ( $back == 1 )  {
      $cmd = $set14;

    } else  {
      $cmd = $set10;

    }

  } elsif ( $selected == $length )  {
    $return_line=4;
    $line1 = sprintf "%c%s", 32, fit ( @list [ $selected - 2 ], 19 );
    $line2 = sprintf "%c%s", 32, fit ( @list [ $selected - 1 ], 19 );
    $line3 = sprintf "%c%-19s", $pointer_char, display_scroll ( @$scr_line );
    if ( $back == 1 )  {
      $cmd = $set13;

    } else  {
      $cmd = $set9;

    }

  } else  {
    $return_line=3;
    $line1 = sprintf "%c%s", 32, fit ( @list [ $selected - 1 ], 19 );
    $line2 = sprintf "%c%-19s", $pointer_char, display_scroll ( @$scr_line );
    $line3 = sprintf "%c%s", 32, fit ( @list [ $selected + 1 ], 19 );
    if ( $back == 1 )  {
      $cmd = $set15;

    } else  {
      $cmd = $set11;

    }

  }

  if ( $display_type eq "crystalfontz" )  {
    display_position ( 17, 1 );
    display_write ( $cmd );

  } elsif ( $display_type ne "console" )  {
    display_write ( $cmd );
    display_time ( );

  }

  display_position ( 1, 2 );

  if ( $display_type eq "crystalfontz" ) {
    display_write ( sprintf ( "%s%c%c%s%c%c%s%", $line1, 10, 13, $line2, 10, 13, $line3 ) );

  } else {
    display_write ( "$line1\n$line2\n$line3\n" );

  }

  return $return_line;

}

#
# menu_render_selected ( \@menu_items, \@tagged, $position );
#   takes a refrence to an array of menu items and and a refrence to an
#   array of tags. displays the menu centered around the $position marker.
#   you have to pass refrences to an array because arguments to a subroutine
#   in perl come in as one array. \@menu is a refrence to @menu.
#

sub menu_render_selected  {

  my ( $list, $tagged, $scr_line, $pos ) = @_;

  my ( $length, $x );

  $length = @$list;
  $length --;

  if ( $length == -1 )  {
    print "select menu list doesn't have any elements\n";

  } elsif ( $length == 0 )  {
    $return_line=2;
    if ( @$tagged [ 0 ] )  {
      $line1 = sprintf "%c%c%-18s", $pointer_char, $select_char, display_scroll ( @$scr_line );

    } else  {
      $line1 = sprintf "%c%c%-18s", $pointer_char, 32, display_scroll ( @$scr_line );

    }
    $line2 = $blank;
    $line3 = $blank;
    $cmd   = $set12;

  } elsif ( $length == 1 )  {
    if ( $pos == 0 )  {
      $return_line=2;
      if ( @$tagged [ 0 ] )  {
        $line1 = sprintf "%c%c%-18s", $pointer_char, $select_char, display_scroll ( @$scr_line );

      } else  {
        $line1 = sprintf "%c%c%-18s", $pointer_char, 32, display_scroll( @$scr_line );

      }
      if ( @$tagged [ 1 ] )  {
        $line2 = sprintf "%c%c%s", 32, $select_char, fit ( @$list [ 1 ], 18 );

      } else  {
        $line2 = sprintf "%c%c%s", 32, 32, fit ( @$list [ 1 ], 18 );

      }
      $line3 = $blank;
      $cmd   = $set14;

    } else  {
      $return_line=3;
      if ( @$tagged [ 0 ] )  {
        $line1 = sprintf "%c%c%s", 32, $select_char, fit ( @$list [ 0 ], 18 );

      } else  {
        $line1 = sprintf "%c%c%s", 32, 32, fit ( @$list [ 0 ], 18 );

      }
      if ( @$tagged [ 1 ] )  {
        $line2 = sprintf "%c%c%-18s", $pointer_char, $select_char, display_scroll ( @$scr_line );

      } else  {
        $line2 = sprintf "%c%c%-18s", $pointer_char, 32, display_scroll ( @$scr_line );

      }
      $line3 = $blank;
      $cmd = $set13;

    }

  } elsif ( $pos == 0 )  {         # cursor is at top
    $return_line=2;
    if ( @$tagged [ 0 ] )  {
      $line1 = sprintf "%c%c%-18s", $pointer_char, $select_char, display_scroll ( @$scr_line );

    } else  {
      $line1 = sprintf "%c%c%-18s", $pointer_char, 32, display_scroll ( @$scr_line );

    }
    if ( @$tagged [ 1 ] )  {
      $line2 = sprintf "%c%c%s", 32, $select_char, fit ( @$list [ 1 ], 18 );

    } else  {
      $line2 = sprintf "%c%c%s", 32, 32, fit ( @$list [ 1 ], 18 );

    }
    if ( @$tagged [ 2 ] )  {
      $line3 = sprintf "%c%c%s", 32, $select_char, fit ( @$list [ 2 ], 18 );

    } else  {
      $line3 = sprintf "%c%c%s", 32, 32, fit ( @$list [ 2 ], 18 );

    }
    $cmd = $set14;

  } elsif ( $pos == $length )  {   # cursor is at the bottom
    $return_line=4;

    if ( @$tagged [ $pos - 2 ] )  {
      $line1 = sprintf "%c%c%s", 32, $select_char, fit ( @$list [ $pos - 2 ], 18 );

    } else  {
      $line1 = sprintf "%c%c%s", 32, 32, fit ( @$list [ $pos - 2 ], 18 );

    }
    if ( @$tagged [ $pos - 1 ] )  {
      $line2 = sprintf "%c%c%s", 32, $select_char, fit ( @$list [ $pos - 1 ], 18 );

    } else  {
      $line2 = sprintf "%c%c%s", 32, 32, fit ( @$list [ $pos - 1 ], 18 );

    }
    if ( @$tagged [ $pos ] )  {
      $line3 = sprintf "%c%c%-18s", $pointer_char, $select_char, display_scroll ( @$scr_line );

    } else  {
      $line3 = sprintf "%c%c%-18s", $pointer_char, 32, display_scroll ( @$scr_line );

    }
    $cmd = $set13;

  } else  {                        # cursor is in the middle
    $return_line=3;
    if ( @$tagged [ $pos - 1 ] )  {
      $line1 = sprintf "%c%c%s", 32, $select_char, fit ( @$list [ $pos - 1 ], 18 );

    } else  {
      $line1 = sprintf "%c%c%s", 32, 32, fit ( @$list [ $pos - 1 ], 18 );

    }
    if ( @$tagged [ $pos ] )  {
      $line2 = sprintf "%c%c%-18s", $pointer_char, $select_char, display_scroll ( @$scr_line );

    } else  {
      $line2 = sprintf "%c%c%-18s", $pointer_char, 32, display_scroll ( @$scr_line );

    }
    if ( @$tagged [ $pos + 1 ] )  {
      $line3 = sprintf "%c%c%s", 32, $select_char, fit ( @$list [ $pos + 1 ], 18 );

    } else  {
      $line3 = sprintf "%c%c%s", 32, 32, fit ( @$list [ $pos + 1 ], 18 );

    }
    $cmd = $set15;

  }

  display_time ( );

  if ( $display_type ne "console" ) {
    display_position ( 17, 1 ) if ( $display_type eq "crystalfontz" );
    display_write ( $cmd );
  }

  display_position ( 1, 2 );

  if ( $display_type eq "crystalfontz" ) {
    display_write ( sprintf ( "%s%c%c%s%c%c%s%", $line1, 10, 13, $line2, 10, 13, $line3 ) );

  } else {
    display_write ( "$line1\n$line2\n$line3\n" );

  }

  return $return_line;


}

#
# menu_navigate ( @menu_items, $position, $back_option );
#   deals with menu selection. returns the position of the pointer
#   when the select key is pressed. if the back key is pressed, the
#   return value is set to -1. $back_option is a boolean.
#

sub menu_navigate  {

  my ( @menu ) = @_;

  my ( $key, $exit, $pos, $back );

  $back = pop ( @menu );
  $pos  = pop ( @menu );
  $exit = 0;

  @pos_scroll = menu_scroll ( $menu[$pos], 19 );

  $line_pos = menu_render ( @menu, \@pos_scroll, $back, $pos );

  while ( !$exit )  {
    $key = display_check_for_key ( .01 );

    display_time ( );

    if ( $key ) {
      if ( $key eq $up_key )  {
        $pos-- unless ( $pos eq 0 );
        @pos_scroll = menu_scroll ( $menu[$pos], 19 );
        $line_pos = menu_render ( @menu, \@pos_scroll, $back, $pos );

      } elsif ( $key eq $down_key )  {
        $pos++ unless ( $pos eq ( @menu - 1 ) );
        @pos_scroll = menu_scroll ( $menu[$pos], 19 );
        $line_pos = menu_render ( @menu, \@pos_scroll, $back, $pos );

      } elsif ( $key eq $back_key )  {
        if ( $back )  {
          $pos  = -1;
          $exit = 1;

        }

      } elsif ( $key eq $forward_key )  {
        $exit = 1;

      }

    } else {
      $pos_scroll[7]++;
      display_scroll_code( \@pos_scroll );
      if ( !$pos_scroll[8] ) {
        if ( length ( $pos_scroll[1]) > 19 ) {
          display_scroll_counter( \@pos_scroll );
          display_position ( 2, $line_pos );
          display_write ( display_scroll ( @pos_scroll ) );

        }

      }

    }

  }

  return ( $pos );

}

#
# $result = menu_text_entry ( "Enter a name...", 20 );
#   displays a text entry screen. quite slow and clutzy, but i didn't want
#   to have a keyboard in my installation. in the example above, the first
#   argument is the queue text and the 20 is the total number of characters.
#

sub menu_text_entry  {

  my ( $prompt, $max ) = @_;

  @characters = ( "Z", "Y", "X", "W", "V", "U", "T", "S", "R", "Q", "P", 
                  "O", "N", "M", "L", "K", "J", "I", "H", "G", "F", "E",
                  "D", "C", "B", "A", " ", "a", "b", "c", "d", "e", "f",
                  "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q",
                  "r", "s", "t", "u", "v", "w", "x", "y", "z", "0", "1", 
                  "2", "3", "4", "5", "6", "7", "8", "9", ".", ",", "-", 
                  ":", ";" );

  my $char_total = @characters;

  my $pos = $exit = $x = 0;
  my @string, $out;

  $max--;

  while ( $x <= $max )  {
    @string[$x] = 26;
    $x++;

  }

  display_position ( 1, 2 );
  display_write ( sprintf "%c%s", 32, fit ( $prompt, 19 ) );
  display_position ( 1, 3 );
  display_write ( sprintf "%20c", 32 );
  display_position ( 1, 4 );
  display_write ( sprintf "%20c", 32 );

  while ( !$exit )  {
    position_cursor ( ( $pos + 1 ) );
    display_position ( ( 1 + $pos ), 3 );
    display_write ( "@characters[@string[$pos]]" );

    $key = display_check_for_key ( 10 );

    if ( $key eq $up_key )  {
      if ( @string[$pos] == -1 )  {
        @string[$pos] = ( $char_total - 1 );

      } else  {
        @string[$pos]--;

      }

    } elsif ( $key eq $down_key )  {
      if ( @string[$pos] == ( $char_total - 1 ) )  {
        @string[$pos] = 0;

      } else  {
        @string[$pos]++;

      }

    } elsif ( $key eq $back_key )  {
      if ( $pos == 0 )  {
        $exit = 1;

      } else  {
        $pos--;

      }

    } elsif ( $key eq $forward_key )  {
      if ( $pos == $max )  {
        $exit = 1;

      } else  {
        $pos++;

      }

    }

  }
  $x = 0;

  while ( $x <= $max )  {
    $out = $out . $characters[@string[$x]];
    $x ++;

  }

  if ( $display ne "console" ) {
    display_blink_off ( );

  }

  return ( $out );

}

#
# position_cursor ( $position );
#   places an edit cursor on the bottom line at the defined position
#

sub position_cursor
{

  my ( $pos ) = @_;

  display_position ( $pos , 4 );

  if ( $pos == 1 )  {
    display_position ( 1, 4 );
    display_write ( sprintf "%c%c", $position_char, 32 );

  } elsif ( $pos == 20 )  {
    display_position ( 18, 4 );
    display_write ( sprintf "%c%c", 32, $position_char );

  } else  {
    display_position ( ( $pos - 1 ), 4 );
    display_write ( sprintf "%c%c%c", 32, $position_char, 32 );

  }

}

#
# menu_select ( \@menu_items, \@tags, $position_in_list );
#   deals with drawing a menu that will let you select one or more items.
#   pointers to arrays are used instead of the arrays themselves because
#   you can't pass two or more arrays easily in perl. because it is a
#   pointer to an array, the origional array is changed from this routine.
#

sub menu_select  {

  my ( $menu, $tagged, $pos ) = @_;

  $exit = 0;

  @pos_scroll = menu_scroll ( $$menu[$pos], 18 );

  $line_pos = menu_render_selected ( $menu, $tagged, \@pos_scroll, $pos );

  while ( !$exit )  {

    select( undef, undef, undef, .01) if ($display_type ne "console");
    $key = display_check_for_key ( .01 );

    display_time ( );

    if ( $key ) {
      if ( $key eq $up_key )  {
        $pos--;
        $pos=( @$menu - 1 ) if ( $pos < 0 );
        @pos_scroll = menu_scroll ( $$menu[$pos], 18 );
        $line_pos = menu_render_selected ( $menu, $tagged, \@pos_scroll, $pos );

      } elsif ( $key eq $down_key )  {
        $pos++;
        $pos=0 if ( $pos > ( @$menu - 1 ) );
        @pos_scroll = menu_scroll ( $$menu[$pos], 18 );
        $line_pos = menu_render_selected ( $menu, $tagged, \@pos_scroll, $pos );

      } elsif ( $key eq $back_key )  {
        $exit = 1;

      } elsif ( $key eq $forward_key )  {
        if ( @$tagged [ $pos ] == 0 )  {
          @$tagged [ $pos ] = 1;

        } else  {
          @$tagged [ $pos ] = 0;

        }
        $line_pos = menu_render_selected ( $menu, $tagged, \@pos_scroll, $pos );

      }

    } else {
      display_scroll_code( \@pos_scroll );
      $pos_scroll[7]++;
      if ( !$pos_scroll[8] ) {
        if ( length ( $pos_scroll[1]) > 18 ) {
          display_scroll_counter( \@pos_scroll );
          display_position ( 3, $line_pos );
          display_write ( display_scroll ( @pos_scroll ) );

        }

      }

    }

  }

  return ( $pos );

}

#
# $result = menu_yes_no ( "Continue?", 1 );
#   displays a yes/no screen and sets the result to the boolean entered. a 
#   prompt and a default value are passed. ( 0 = yes, 1 = no )
#

sub menu_yes_no  {

  my ( $prompt, $pos ) = @_;
  my $exit = 0;

  display_position ( 1, 2 );
  display_write ( fit ( $prompt, 20 ) );

  while ( !$exit )  {
    display_position ( 1, 3 );
    if ( $pos == 0 )  {
      display_write ( sprintf "%c%s%-16c", $pointer_char, "Yes", 32 );

    } else  {
      display_write ( sprintf "%c%s%-16c", 32, "Yes", 32 );

    }
    display_position ( 1, 4 );
    if ( $pos == 1 )  {
      display_write ( sprintf "%c%s%-17c", $pointer_char, "No", 32 );

    } else  {
      display_write ( sprintf "%c%s%-17c", 32, "No", 32 );

    }

    $key = display_check_for_key ( 1 );

    display_time ( );

    if ( ( $key eq $up_key ) || ( $key eq $down_key ) )  {
      if ( $pos == 0 )  {
        $pos = 1;

      } else  {
        $pos = 0;

      }

    } elsif ( $key eq $forward_key )  {
      $exit = 1;

      if ( $pos == 0 )  {
        return ( 1 );

      } else  {
        return ( 0 );

      }

    }

  }

}

#
# menu_info ( "User added.", .5 );
#   displays an info screen and sleeps for the specified seconds.
#

sub menu_info  {

  my ( $prompt, $delay ) = @_;

  display_position ( 1, 2 );
  display_write ( fit ( " ", 20 ) );
  display_position ( 1, 3 );
  display_write ( center_fit ( $prompt, 20 ) );
  display_position ( 1, 4 );
  display_write ( fit ( " ", 20 ) );

  select ( undef, undef, undef, $delay );

}

#
# menu_mini ( $left_text, $middle_text, $right_text );
#   renders a nice little line with 3 elements
#

sub menu_mini  {

  my ( $left, $middle, $right ) = @_;

  if ( $display_type eq "vfd" ) {
    $char = sprintf ("%c", 0x07);

  } else {
    $char = sprintf ("%c", 128);

  }

  my $mid_len = length ( $middle ) + 1;
  my $left_offset = $right_offset = 10 - int( $mid_len / 2 );

  $right_offset -- if ( ( $mid_len / 2 ) != int( $mid_len / 2 ) );

  $mini_out = fit ( ( sprintf "%c%s", $back_char, $left ), $left_offset );
  $mini_out = $mini_out . $char . $middle;
  $mini_out = $mini_out . reverse_fit ( ( sprintf "%s%c", $right, $pointer_char ), $right_offset );

  display_position ( 1, 4 );
  display_write ( $mini_out );

}

#
# menu_until_keypress ( "Press a Key" );
#  routine to hang out until the user presses any key
#

sub menu_until_keypress  {

  my ( $prompt ) = @_;

  my $length = length ( $prompt );
  my $start = 10 - int( $length / 2 );
  my $exit = false;
  my $key = "";

  display_position ( 1, 2 );
  display_write ( fit ( " ", 20 ) );
  display_position ( 1, 3 );
  display_write ( center_fit ( $prompt, 20 ) );
  display_position ( 1, 4 );
  display_write ( center_fit ( "Press a Key to Exit", 20 ) );

  while ( !$exit )  {
    $key = display_check_for_key ( 10 );

    if ( $key ne "" )  {
      $exit = true;

    }

  }

}

sub menu_scroll {
  my ( $m_text, $max_len ) = @_;

  if ( length ( $m_text ) > $max_len ) {
    $scroll_text=" " . $m_text . " ***" if ( $ln_scr_mode eq "marquee" );
    $scroll_text="*  $m_text  *" if ( $ln_scr_mode eq "bounce" );

  } else {
    $scroll_text=$m_text;

  }

  return ( 0, $scroll_text, 0, $max_len, $ln_scr_mode, "r", $ln_scr_prc, 0, 0 );

}

#
# the following lines build a string for the special menu navigation options
# character that is used between the hour and minutes in the clock.
#

$set0 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00;
$set1 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x04, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00;
$set2 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x04, 0x00;
$set3 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x04, 0x04, 0x00, 0x00, 0x00, 0x04, 0x04, 0x00;
$set4 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x00, 0x00, 0x00, 0x18, 0x00, 0x00, 0x00, 0x00;
$set5 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x04, 0x04, 0x00, 0x18, 0x00, 0x00, 0x00, 0x00;
$set6 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x00, 0x00, 0x00, 0x18, 0x00, 0x04, 0x04, 0x00;
$set7 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x04, 0x04, 0x00, 0x18, 0x00, 0x04, 0x04, 0x00;

if ( $display_type eq "crystalfontz" ) {
  $set8 = " "; 
  $set9 = sprintf "%c", 129; 
  $set10 = sprintf "%c", 130; 
  $set11 = sprintf "%c", 131; 
  $set12 = sprintf "%c", 132; 
  $set13 = sprintf "%c", 133; 
  $set14 = sprintf "%c", 134; 
  $set15 = sprintf "%c", 135; 

} else {

  $set8 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00;
  $set9 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x04, 0x04, 0x00, 0x03, 0x00, 0x00, 0x00, 0x00;
  $set10 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x00, 0x00, 0x00, 0x03, 0x00, 0x04, 0x04, 0x00;
  $set11 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x04, 0x04, 0x00, 0x03, 0x00, 0x04, 0x04, 0x00;
  $set12 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x00, 0x00, 0x00, 0x1b, 0x00, 0x00, 0x00, 0x00;
  $set13 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x04, 0x04, 0x00, 0x1b, 0x00, 0x00, 0x00, 0x00;
  $set14 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x00, 0x00, 0x00, 0x1b, 0x00, 0x04, 0x04, 0x00;
  $set15 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x00,
                0x04, 0x04, 0x00, 0x1b, 0x00, 0x04, 0x04, 0x00;

}
