drop database route66;
create database route66;
use mysql;
grant all privileges on route66.* to route66@localhost identified by 'route66';
flush privileges;
use route66;
CREATE TABLE versions (
    version         varchar(32)
);

INSERT INTO versions (
    version
) VALUES (
    '0.6.0'
);

CREATE TABLE usernames (
    usernameid      int unsigned not null auto_increment primary key,
    username        varchar(64) not null
);

CREATE TABLE songprefs (
    songid          int unsigned not null auto_increment,
    usernameid      int not null,
    preference      float not null,
    index           songprefs_index (preference),
    primary key     (songid, usernameid)
);

CREATE TABLE artistprefs (
    artistid        int unsigned not null auto_increment,
    usernameid      int not null,
    preference      float not null,
    index           artistprefs_index (preference),
    primary key     (artistid, usernameid)
);

CREATE TABLE albumprefs (
    albumid         int unsigned not null auto_increment,
    usernameid      int not null,
    preference      float not null,
    index           albumprefs_index (preference),
    primary key     (albumid, usernameid)
);

CREATE TABLE genreprefs (
    genreid         int unsigned not null auto_increment,
    usernameid      int not null,
    preference      float not null,
    index           genreprefs_index (preference),
    primary key     (genreid, usernameid)
);

CREATE TABLE songs (
    songid          int unsigned not null auto_increment primary key,
    mood            float not null,
    tempo           float not null,
    endsilence      float,                       -- in seconds
    tracknumber     int not null,
    year            int not null,
    song            varchar(64) not null,
    index           songs_index (mood, tempo, year, song)
);

CREATE TABLE artists (
    artistid        int unsigned not null auto_increment primary key,
    artist          varchar(64) not null,
    index           artists_index (artist)
);

CREATE TABLE albums (
    albumid         int unsigned not null auto_increment,
    artistid        int not null,
    album           varchar(64) not null,
    year            int not null,
    index           albums_index (album, year),
    primary key     (albumid, artistid)
);

CREATE TABLE filesystems (
    filesystemid    int unsigned not null auto_increment primary key,
    filesystem      varchar(64) not null
);

CREATE TABLE genres (
    genreid         int unsigned not null auto_increment primary key,
    genre           varchar(64) not null,
    index           genres_index (genre)
);

CREATE TABLE contributors (
    contributorid   int unsigned not null auto_increment primary key,
    contributor     varchar(64) not null,
    index           contributors_index (contributor)
);

CREATE TABLE paths (
    pathid          int unsigned not null auto_increment,
    filesystemid    int not null,
    contributorid   int not null,
    artistid        int not null,
    albumid         int not null,
    songid          int not null,
    genreid         int not null,
    volume          float not null,
    timestamp       datetime,
    path            varchar(255) not null,
    obsolete        enum("t", "f") not null,
    index           paths_index (obsolete),
    primary key     (pathid, filesystemid, contributorid, artistid, albumid, songid, genreid)
);

CREATE TABLE history (
    usernameid      int not null,
    pathid          int not null,
    timestamp       datetime
);

CREATE TABLE playlists (
    playlistid      int unsigned not null auto_increment primary key,
    playlist        varchar(64) not null,
    index           playlists_index (playlist)
);

CREATE TABLE playlistitems (
    playlistitemid  int unsigned not null auto_increment primary key,
    playlistid      int not null,
    item            varchar(64) not null
);

CREATE TABLE current (
    usernameid      int not null,
    pathid          int not null,
    playlistid      int not null
);

INSERT INTO current (
    usernameid,     pathid,
    playlistid
) VALUES (
    0,              0,
    0
);

--
-- default data
--

INSERT INTO filesystems (
    filesystemid, filesystem
) VALUES (
    null, 'Default Filesystem'
);

INSERT INTO usernames (
    usernameid, username
) VALUES (
    null, 'Default'
);

INSERT INTO contributors (
    contributorid, contributor
) VALUES (
    null, 'Default'
);
