#
# route66 database access module
#   by Anders Brownworth (anders-route66@evantide.com)
#   No Editable Section in here ...  Look in Settings.pm
#

package Database;

use Exporter;
use DBI;

$db          = "mysql";
$SQL_MAX_INT = 2147483647;

@ISA	= qw ( Exporter );
@EXPORT	= qw ( database_open database_get_filesystem database_put_filesystem
	       database_get_current database_update_current
               database_get_song database_put_song database_get_username
               database_put_username database_get_all_usernames 
               database_get_artistid database_put_artist database_put_album
               database_get_all_albums database_put_genre
               database_get_all_genres database_get_all_artists
               database_get_genreid database_get_albumid database_put_path 
               database_raw_sql database_get_path database_get_playlist
               database_get_album database_get_artist database_set_volume 
               database_get_usernameid database_get_preference
               database_get_songid database_update_preference
               database_put_history database_put_contributor 
               database_get_filesystems database_is_path_inserted 
               database_update_artist database_update_album
               database_update_genre database_get_songid_from_song 
               database_update_song );

#
# escape ( $line );
#   escapes ticks, newlines and tabs to keep database insertion code legal
#

sub escape
{

  @_[0] =~ s/'/\\'/g;
  @_[0] =~ s/\n/\\n/g;
  @_[0] =~ s/\t/\\t/g;

}

#
# print "looks good\n" unless limit ( $value, $min, $max );
#

sub limit  {

  my ( $num, $min, $max ) = @_;

  if ( ( $num <= $max ) && ( $num >= $min ) )  {
    return ( false );

  }

}

#
# ( $database_handle, $version ) = database_open ( );
#   opens a connection to the database, looks up the database version
#   number and returns the database handle and the version text.
#

sub database_open
{
    my ( $dbh );

    $dbh = DBI->connect("dbi:$db:dbname=route66", "route66", "route66")
            or die "Cannot connect to database: $DBI::errstr";
    $sth = $dbh->prepare ( "select version from versions" )
            or die "Cannot prepare open SQL: $DBI::errstr";
    $sth->execute or die "Cannot execute open SQL: $DBI::errstr";

    $row = $sth->fetchrow_array;

    return ( $dbh, $row );
}

#
# $filesystem = database_get_filesystem ( $database_handle, $filesystemid );
#   returns filesystem names from filesystem id numbers.
#

sub database_get_filesystem
{

  my ( $dbh, $filesystemid ) = @_;

  $sth = $dbh->prepare ( "select filesystem from filesystems
                          where ( filesystemid = $filesystemid )" )
          or die "Can't prepare get_filesystem SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_filesystem SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# $filesystemid = database_put_filesystem ( $database_handle, 
#                                           $filesystem_name );
#   creates a record for a filesystem and returns the id
#

sub database_put_filesystem
{

  my ( $dbh, $filesystem ) = @_;

  escape ( $filesystem );

  $sth = $dbh->prepare ( "insert into filesystems ( filesystemid,
                          filesystem ) values ( 
                          null, '$filesystem' )" )
          or die "Can't prepare put_filesystem SQL: $DBI::errstr";
  $sth->execute or die "Can't execute put_filesystem SQL: $DBI::errstr";
  $id = $dbh->{mysql_insertid};

  return ( $id );

}

#
# ( $usernameid, $pathid, $playlistid ) = 
#                database_get_current ( $database_handle );
#   returns the first entry for usernameid and pathid from the current table.
#

sub database_get_current
{

  my ( $dbh ) = @_;

  $sth = $dbh->prepare ( "select usernameid, pathid, playlistid from current" )
          or die "Can't prepare get_current SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_current SQL: $DBI::errstr";

  @row_ary = $sth->fetchrow_array;

  return ( @row_ary );

}

#
# database_update_current ( $database_handle, $usernameid, $pathid,
#                           $playlistid );
#   updates the current table.
#

sub database_update_current
{

  my ( $dbh, $usernameid, $pathid, $playlistid ) = @_;

  $sth = $dbh->prepare ( "update current set
                          usernameid = '$usernameid',
                          pathid = '$pathid',
                          playlistid = '$playlistid'" )
          or die "Can't prepare update_current SQL: $DBI::errstr";
  $sth->execute or die "Can't execute update_current SQL: $DBI::errstr";

}

#
# ( $mood, $tempo, $endsilence, $tracknumber, $year, $song ) = 
#                  database_get_song ( $database_handle, $songid );
#   get mood, tempo, end silence, track number and song name from songid.
#

sub database_get_song
{

  my ( $dbh, $songid ) = @_;

  $sth = $dbh->prepare ( "select mood, tempo, endsilence, 
                          tracknumber, year, song from songs
                          where songid = '$songid'" )
          or die "Can't prepare get_song SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_song SQL: $DBI::errstr";

  @row_ary = $sth->fetchrow_array;

  return ( @row_ary );

}

#
# $songid = database_put_song ( $database_handle, $mood, $tempo, 
#                               $endsilence, $tracknumber, $year, $song );
#   insert a song into the database
#

sub database_put_song
{

  my ( $dbh, $mood, $tempo, $endsilence, $tracknumber, $year, $song ) = @_;

  if ( limit ( $mood, 0, 1 ) && limit ( $tempo, 0, 1 ) )  {
    escape ( $song );

    $sth = $dbh->prepare ( "insert into songs ( songid, mood, tempo,
                            endsilence, tracknumber, year, song ) values (
                            null, '$mood', '$tempo', 
                            '$endsilence', '$tracknumber', '$year', 
                            '$song' )" )
                  or die "Can't prepare put_song SQL: $DBI::errstr";
    $sth->execute or die "Can't execute put_song SQL: $DBI::errstr";

    $id = $dbh->{mysql_insertid};   

    return ( $id );

  }
  else  {
    print "NOT INSERTED! (mood=$mood tempo=$tempo) value out of range\n";

  }

}

#
# $username = database_get_username ( $database_handle, $usernameid );
#   returns a username for a usernameid
#

sub database_get_username
{

  my ( $dbh, $usernameid ) = @_;

  $sth = $dbh->prepare ( "select username from usernames
                          where ( usernameid = $usernameid )" )
          or die "Can't prepare get_username SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_username SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# $usernameid = database_get_usernameid ( $database_handle, $username );
#   returns a usernameid for a username
#

sub database_get_usernameid
{

  my ( $dbh, $username ) = @_;

  $sth = $dbh->prepare ( "select usernameid from usernames
                          where ( username = '$username' )" )
          or die "Can't prepare get_usernameid SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_usernameid SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# $usernameid = database_put_username ( $database_handle, $username );
#   inserts a username into the database and returns the usernameid.
#

sub database_put_username
{

  my ( $dbh, $username ) = @_;

  escape ( $username );

  $sth = $dbh->prepare ( "insert into usernames ( usernameid, username )
                          values ( null, '$username' )" )
          or die "Can't prepare put_username SQL: $DBI::errstr";
  $sth->execute or die "Can't execute put_username SQL: $DBI::errstr";

  $id = $dbh->{mysql_insertid};   

  return ( $id );

}

#
# ( @usernames ) = database_get_all_usernames ( $database_handle );
#   returns an array of all the usernames in the database
#

sub database_get_all_usernames
{

  my ( $dbh, $usernameid ) = @_;
  my @ary;

  $ary_ref = $dbh->selectall_arrayref ( "select username from usernames" );

  foreach $item ( @$ary_ref )  {
    push @ary, $item->[0];

  }

  return ( @ary );

}

#
# $artistid = database_get_artistid ( $database_handle, $artist );
#   returns an artistid for an artist name
#

sub database_get_artistid
{

  my ( $dbh, $artist ) = @_;

  escape ( $artist );

  $sth = $dbh->prepare ( "select artistid from artists
                          where ( artist = '$artist' )" )
          or die "Can't prepare get_artistid SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_artistid SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# $artistid = database_put_artist ( $database_handle, $artist );
#   adds an artist to the database and returns the artistid
#

sub database_put_artist
{

  my ( $dbh, $artist ) = @_;

  escape ( $artist );

  $sth = $dbh->prepare ( "insert into artists ( artistid, artist )
                          values ( null, '$artist' )" )
          or die "Can't prepare put_artist SQL: $DBI::errstr";
  $sth->execute or die "Can't execute put_artist SQL: $DBI::errstr";

  $id = $dbh->{mysql_insertid};

  return ( $id );

}

#
# $albumid = database_put_album ( $database_handle, $album, $artistid, $year );
#   adds an album to the database and returns the albumid
#

sub database_put_album
{

  my ( $dbh, $album, $artistid, $year ) = @_;

  escape ( $album );

  $sth = $dbh->prepare ( "insert into albums ( 
                            albumid, artistid, album, year
                          ) values (
                            null, $artistid,
                            '$album', $year )" )
          or die "Can't prepare put_album SQL: $DBI::errstr";
  $sth->execute or die "Can't execute put_album SQL: $DBI::errstr";

  $id = $dbh->{mysql_insertid};

  return ( $id );

}

#
# ( @albums ) = database_get_all_albums ( $database_handle );
#   returns an array of all the albums in the database
#

sub database_get_all_albums
{

  my ( $dbh, $albumid ) = @_;
  my @ary;

  $ary_ref = $dbh->selectall_arrayref ( "select album from albums" );

  foreach $item ( @$ary_ref )  {
    push @ary, $item->[0];

  }

  return ( @ary );

}

#
# $genreid = database_put_genre ( $database_handle, $genre );
#   adds an genre to the database and returns the genreid
#

sub database_put_genre
{

  my ( $dbh, $genre ) = @_;

  escape ( $genre );

  $sth = $dbh->prepare ( "insert into genres ( genreid, genre )
                          values ( null, '$genre' )" )
          or die "Can't prepare put_genre SQL: $DBI::errstr";
  $sth->execute or die "Can't execute put_genre SQL: $DBI::errstr";

  $id = $dbh->{mysql_insertid};

  return ( $id );

}

#
# ( @genres ) = database_get_all_genres ( $database_handle );
#   returns an array of all the genres in the database
#

sub database_get_all_genres
{

  my ( $dbh, $genreid ) = @_;
  my @ary;

  $ary_ref = $dbh->selectall_arrayref ( "select genre from genres" );

  foreach $item ( @$ary_ref )  {
    push @ary, $item->[0];

  }

  return ( @ary );

}

#
# ( @artists ) = database_get_all_artists ( $database_handle );
#   returns an array of all the artists in the database
#

sub database_get_all_artists
{

  my ( $dbh, $artistid ) = @_;
  my @ary;

  $ary_ref = $dbh->selectall_arrayref ( "select artist from artists" );

  foreach $item ( @$ary_ref )  {
    push @ary, $item->[0];

  }

  return ( @ary );

}

#
# $genreid = database_get_genreid ( $database_handle, $genre );
#   returns an genreid for an genre name
#

sub database_get_genreid
{

  my ( $dbh, $genre ) = @_;

  escape ( $genre );

  $sth = $dbh->prepare ( "select genreid from genres
                          where ( genre = '$genre' )" )
          or die "Can't prepare get_genreid SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_genreid SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# $albumid = database_get_albumid ( $database_handle, $album );
#   returns an albumid for an album name
#

sub database_get_albumid
{

  my ( $dbh, $album ) = @_;

  escape ( $album );

  $sth = $dbh->prepare ( "select albumid from albums
                          where ( album = '$album' )" )
          or die "Can't prepare get_albumid SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_albumid SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# $pathid = database_put_path ( $database_handle, $filesystemid,
#                               $contributorid, $artistid, $songid,
#                               $albumid, $genreid, $volume, $path );
#   creates a record for a path
#

sub database_put_path
{

  my ( $dbh, $filesystemid, $contributorid, $artistid, $songid,
       $albumid, $genreid, $volume, $path ) = @_;

  if ( limit ( $volume, 0, 100 ) )  {
    $volume = $volume / 100;

    escape ( $path );

    $sth = $dbh->prepare ( "insert into paths ( 
                            pathid, filesystemid, contributorid, artistid,
                            albumid, songid, genreid, volume, timestamp,
                            path, obsolete
                          ) values (
                            null, $filesystemid, 
                            $contributorid, $artistid, $albumid, $songid,
                            $genreid, $volume, NOW(), '$path', 'f' )" )
               or die "Can't prepare put_path SQL: $DBI::errstr";
    $sth->execute or die "Can't execute put_path SQL: $DBI::errstr";

    $id = $dbh->{mysql_insertid};

    return ( $id );

  }
  else  {
    print "NOT INSERTED! (volume=$volume) value out of range\n";

  }

}

#
# ( @results ) = database_raw_sql ( $database_handle, $sql_query );
#   returns an array of the first elements returned by the sql query given.
#   note, this will not return multiple columns. just the first one.
#
 
sub database_raw_sql
{
 
  my ( $dbh, $sql ) = @_;
  my @ary;

#  print "$sql\n";

  $ary_ref = $dbh->selectall_arrayref ( "$sql" );

  foreach $item ( @$ary_ref )  {
    push @ary, $item->[0];

  }
   
  return ( @ary );

}

#
# ( $filesystemid, $contributorid, $artistid, $albumid, $songid, $genreid
#   $volume, $timestamp, $path ) = database_get_path ( $dbh, $pathid );
#    get all the path info for a pathid
#

sub database_get_path
{

  my ( $dbh, $pathid ) = @_;

  $sth = $dbh->prepare ( "select filesystemid, contributorid, artistid,
                          albumid, songid, genreid, volume, timestamp,
                          path from paths where pathid = '$pathid'" )
          or die "Can't prepare get_path SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_path SQL: $DBI::errstr";

  @row_ary = $sth->fetchrow_array;

  @row_ary[6] = @row_ary[6] * 100 if ( @row_ary[6] < 1 );

  return ( @row_ary );

}

#
# $playlist = database_get_playlist ( $dbh, $playlistid );
#    get the playlist name for a playlistid
#

sub database_get_playlist
{

  my ( $dbh, $playlistid ) = @_;

  $sth = $dbh->prepare ( "select playlist from playlists where
                         playlistid = '$playlistid'" )
          or die "Can't prepare get_playlist SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_playlist SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );
 
}

#
# ( $artistid, $album, $year ) = database_get_album ( $dbh, $albumid );
#    get all the artist info for an albumid
#

sub database_get_album
{

  my ( $dbh, $albumid ) = @_;

  $sth = $dbh->prepare ( "select artistid, album, year from albums 
                          where albumid = '$albumid'" )
          or die "Can't prepare get_album SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_album SQL: $DBI::errstr";

  @row_ary = $sth->fetchrow_array;

  return ( @row_ary );

}

#
# $artist = database_get_artist ( $dbh, $artistid );
#    get the artist name for an artistid
#

sub database_get_artist
{

  my ( $dbh, $artistid ) = @_;

  $sth = $dbh->prepare ( "select artist from artists where
                          artistid = '$artistid'" )
          or die "Can't prepare get_artist SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_artist SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# $songid = database_get_songid ( $dbh, $pathid );
#   get songid from pathid
#

sub database_get_songid
{

  my ( $dbh, $pathid ) = @_;

  $sth = $dbh->prepare ( "select songid from paths where
                          pathid = '$pathid'" )
          or die "Can't prepare get_songid SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_songid SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# $prefrence = database_get_preference ( $dbh, $pathid, $usernameid );
#   returns the prefrence setting for a pathid if one exists. if not, it
#   creates one.

sub database_get_preference
{

  my ( $dbh, $pathid, $usernameid ) = @_;
  my ( $songid ) = database_get_songid ( $dbh, $pathid );

  $sth = $dbh->prepare ( "select preference from songprefs where
                          ( usernameid = '$usernameid' ) and 
                          ( songid = '$songid' )" )
          or die "Can't prepare get_preference SQL : $DBI::errstr";
  $sth->execute or die "Can't execute get_preference SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  if ( $row eq "" )  {
    database_put_preference ( $dbh, $songid, $usernameid, 50 );
    return ( 50 );

  }
  else  {
    return ( $row * 100 );

  }

}

#
# database_set_volume ( $dbh, $pathid, $volume );
#   sets the volume for a pathid
#

sub database_set_volume
{

  my ( $dbh, $pathid, $volume ) = @_;

  if ( limit ( $volume, 0, 100 ) )  {
    $volume = $volume / 100;

    $sth = $dbh->prepare ( "update paths set volume = $volume where
                            pathid = $pathid" )
            or die "Can't prepare set_volume SQL: $DBI::errstr";
    $sth->execute or die "Can't execute set_volume SQL: $DBI::errstr";

  }
  else  {
    print "NOT INSERTED! (volume=$volume) value out of range\n";

  }

}

#
# database_put_preference ( $dbh, $songid, $usernameid, $preference );
#   creates a preference record for a songid for a user
#

sub database_put_preference
{

  my ( $dbh, $songid, $usernameid, $preference ) = @_;

  if ( limit ( $preference, 0, 100 ) )  {
    $preference = $preference / 100;

    $sth = $dbh->prepare ( "insert into songprefs 
                            ( songid, usernameid, preference )
                            values
                            ( '$songid', '$usernameid', '$preference' )" )
            or die "Can't prepare put_preference SQL: $DBI::errstr";
    $sth->execute or die "Can't execute put_preference SQL: $DBI::errstr";

  }
  else  {
    print "NOT INSERTED! (preference=$preference) value out of range\n";

  }

}

#
# database_put_history ( $dbh, $usernameid, $pathid );
#   adds a history record for a particular path and user
#

sub database_put_history
{

  my ( $dbh, $usernameid, $pathid ) = @_;

  $sth = $dbh->prepare ( "insert into history
                          ( usernameid, pathid, timestamp )
                          values
                          ( '$usernameid', '$pathid', NOW() )" )
           or die "Can't prepare put_history SQL: $DBI::errstr";
  $sth->execute or die "Can't execute put_history SQL: $DBI::errstr";

}

#
# database_update_preference ( $dbh, $songid, $usernameid, $preference );
#   update the preference setting for a songid for a usernameid
#

sub database_update_preference
{

  my ( $dbh, $songid, $usernameid, $preference ) = @_;

  if ( limit ( $preference, 0, 100 ) )  {
    $preference = $preference / 100;

    $sth = $dbh->prepare ( "update songprefs set preference = '$preference'
                          where ( usernameid = '$usernameid' and
                                  songid = '$songid' ) " )
            or die "Can't prepare update_preference SQL: $DBI::errstr";
    $sth->execute or die "Can't execute update_preference SQL: $DBI::errstr";

  }
  else  {
    print "NOT UPDATED! (preference=$preference) value out of range\n";

  }

}

#
# $contributorid = database_put_contributor ( $database_handle, $contributor );
#   inserts a contributor into the database and returns the contributorid.  
#
 
sub database_put_contributor
{
 
  my ( $dbh, $contributor ) = @_;

  escape ( $contributor );

  $sth = $dbh->prepare ( "insert into contributors ( contributorid, contributor )
                          values ( null, '$contributor' )" )
          or die "Can't prepare put_contributor SQL: $DBI::errstr";
  $sth->execute or die "Can't execute put_contributor SQL: $DBI::errstr";

  $id = $dbh->{mysql_insertid};

  return ( $id );

}

#
# @filesystems = database_get_filesystems ( $dbh );
#   returns a list of all filesystem ids
#

sub database_get_filesystems
{
 
  my ( $dbh ) = @_;

  $sth = $dbh->prepare ( "select filesystemid from filesystems" )
          or die "Can't prepare get_filesystems SQL: $DBI::errstr"; 
  $sth->execute or die "Can't execute get_filesystems SQL: $DBI::errstr";

  @row_ary = $sth->fetchrow_array;

  return ( @row_ary );

}

#
#
#

sub database_is_path_inserted
{

  my ( $dbh, $path ) = @_;

  $sth = $dbh->prepare ( "select path from paths where path = '$path'" )
          or die "Can't prepare get_filesystems SQL: $DBI::errstr"; 
  $sth->execute or die "Can't execute get_filesystems SQL: $DBI::errstr";

  if ( ( $path ) eq $sth->fetchrow_array )  {
    return 1;

  }
  else  {
    return 0;

  }

}

#
# database_update_artist ( $dbh, $artistid, $artist );
#   updates the artist record
#
 
sub database_update_artist
{

  my ( $dbh, $artistid, $artist ) = @_;

  $sth = $dbh->prepare ( "update artists set artist = '$artist' where
                            artistid = $artistid" )
            or die "Can't prepare update_artist SQL: $DBI::errstr";
  $sth->execute or die "Can't execute update_artist SQL: $DBI::errstr";
 
}

#
# database_update_album ( $dbh, $albumid, $artistid, $album, $year );
#   updates the album record
#
 
sub database_update_album
{

  my ( $dbh, $albumid, $artistid, $album, $year ) = @_;

  $sth = $dbh->prepare ( "update albums set artistid = $artistid, 
        album = '$album', year = $year where albumid = $albumid" )
            or die "Can't prepare update_album SQL: $DBI::errstr";
  $sth->execute or die "Can't execute update_album SQL: $DBI::errstr";
 
}

#
# database_update_genre ( $dbh, $genreid, $genre );
#   updates the genre record
#
 
sub database_update_genre
{

  my ( $dbh, $genreid, $genre ) = @_;

  $sth = $dbh->prepare ( "update genres set genre = '$genre' where
                            genreid = $genreid" )
            or die "Can't prepare update_genre SQL: $DBI::errstr";
  $sth->execute or die "Can't execute update_genre SQL: $DBI::errstr";
 
}

#
# $songid = database_get_songid_from_song ( $dbh, $song );
#   get songid from song name
#
 
sub database_get_songid_from_song
{
 
  my ( $dbh, $song ) = @_;

  $sth = $dbh->prepare ( "select songid from songs where
                          song = '$song'" )
          or die "Can't prepare get_songid SQL: $DBI::errstr";
  $sth->execute or die "Can't execute get_songid SQL: $DBI::errstr";

  $row = $sth->fetchrow_array;

  return ( $row );

}

#
# database_update_song ( $dbh, $songid, $song );
#   updates the song record
#
 
sub database_update_song
{

  my ( $dbh, $songid, $song ) = @_;

  $sth = $dbh->prepare ( "update songs set song = '$song' where
                            songid = $songid" )
            or die "Can't prepare update_song SQL: $DBI::errstr";
  $sth->execute or die "Can't execute update_song SQL: $DBI::errstr";
 
}
