#
# Display.pm
#   Matrix Orbital display driver (40 character by 4 line version)
#   nothing editable in here... look in Settings.pm
#
#   CrystalFontz Display patch by Ryan E. Hammond
#   Console Display patch by Michael J. Kidd
#

package Display;

require Term::Cap;
use Settings;
use Exporter;
use POSIX;
use IO;

@ISA	= qw ( Exporter );
@EXPORT	= qw ( display_init display_write display_clear display_close
	       display_position display_time display_check_for_key 
               display_blink_on display_blink_off display_bargraph
               display_init_bargraph display_scroll_counter 
               display_scroll_code display_scroll );

my ( $display_type, $serial_port,
     $up_key, $down_key, $forward_key, $back_key,
     $setvol_cmd, $menu_scr_mode, $radio_pass_thru, 
     $halt_cmd ) = settings_get ( );

$brightness = 50;
$contrast   = 50;


#
# display_init ( );
#   opens and clears the display
#

sub display_init
{

  if ( $display_type ne "console" ) {
    open ( DISPLAY, "+> $serial_port" ) || die "Couldn't open $serial_port. $!";
    my $baud;

    $fd_disp = fileno ( DISPLAY );

    $termios = POSIX::Termios->new ( );
    $termios->getattr ( $fd_disp );
    if ( $display_type eq "crystalfontz" ) {
      $baud = B9600;

    } else {
      $baud = B19200;

    }
    $termios->setispeed ( $baud );
    $termios->setospeed ( $baud );

    $cflag = $termios->getcflag ( );
    $lflag = $termios->getlflag ( );
    $oflag = $termios->getoflag ( );
    $iflag = $termios->getiflag ( );

    $iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
    $oflag &= ~OPOST;
    $lflag &= ~(ECHO|ECHONL|ICANON|ISIG);
    $cflag &= ~(CSIZE|PARENB|HUPCL);
    $cflag |= CS8|CLOCAL;

    $termios->setcflag ( $cflag );
    $termios->setlflag ( $lflag );
    $termios->setoflag ( $oflag );
    $termios->setiflag ( $iflag );
    $termios->setattr ( $fd_disp, TCSANOW );

    if ( $display_type eq "crystalfontz" ) {
      $reboot      = sprintf ( "         %c%c", 26, 26 );
      $on          = sprintf ( "%c", 3 );
      $scroll_wrap = sprintf ( "%c%c", 20, 24 );
      $clear       = sprintf ( "%c", 12 );
      $hide_cursor = sprintf ( "%c", 4 );
      $dim         = sprintf ( "%c%c%c%c", 14, $brightness, 15, $contrast );

      my $set0 = sprintf ( "%c%c%c%c%c%c%c%c%c%c", 25, 0,  8, 28, 62,  0, 62, 28,  8,  0 );
      my $set1 = sprintf ( "%c%c%c%c%c%c%c%c%c%c", 25, 1,  0,  4,  4,  0,  0,  0,  0, 0 );
      my $set2 = sprintf ( "%c%c%c%c%c%c%c%c%c%c", 25, 2,  0,  0,  0,  0,  4,  4,  0, 0 );
      my $set3 = sprintf ( "%c%c%c%c%c%c%c%c%c%c", 25, 3,  0,  4,  4,  0,  4,  4,  0, 0 );
      my $set4 = sprintf ( "%c%c%c%c%c%c%c%c%c%c", 25, 4,  0,  0,  0, 24,  0,  0,  0,  0 );
      my $set5 = sprintf ( "%c%c%c%c%c%c%c%c%c%c", 25, 5,  0,  4,  4, 24,  0,  0,  0,  0 );
      my $set6 = sprintf ( "%c%c%c%c%c%c%c%c%c%c", 25, 6,  0,  0,  0, 24,  4,  4,  0,  0 ); 
      my $set7 = sprintf ( "%c%c%c%c%c%c%c%c%c%c", 25, 7,  0,  4,  4, 24,  4,  4,  0,  0 );
      $display_init_special = "$set0$set1$set2$set3$set4$set5$set6$set7";

      print DISPLAY "$reboot\n";
      print DISPLAY "$on\n";
      sleep 2;
      print DISPLAY "$display_init_special";
      print DISPLAY "$scroll_wrap";
      print DISPLAY "$hide_cursor";
      print DISPLAY "$clear";
      print DISPLAY "$hide_cursor";
      sleep 1;

    } else {
      $clear   = sprintf ( "%cX", 0xfe );
      $noblink = sprintf ( "%cT", 0xfe );
      $dim     = sprintf ( "%cY%c", 0xfe, 0x03 );
      $on      = sprintf ( "%cB%c", 0xfe, 0x00 );
      $repeat  = sprintf ( "%c~%c", 0xfe, 0x00 );
      print DISPLAY "$on$repeat$noblink$dim$clear";

    }

  } else {
    my $termios = new POSIX::Termios;
    my $ospeed = $termios->getospeed;

    $termios->getattr;
    $terminal = Tgetent Term::Cap { TERM => undef, OSPEED => $ospeed };
    $terminal->Trequire(qw/ce ku kd/);

    system "stty", '-echo';
    system "clear";

  }

}

#
# display_write ( "hello there" );
#   sends the specified text to the display at the current cursor position
#

sub display_write
{

  my ( $text ) = @_;

  if ( $display_type ne "console" ) {
    if ( $display_type eq "crystalfontz" ) {
      print DISPLAY $text;

    } else {
      print DISPLAY "$text\n";

    }

  } else {
    print "$text\n";

  }

}

#
# display_clear ( );
#   clears the display
#

sub display_clear
{

  if ($display_type ne "console") {
    if ( $display_type eq "crystalfontz" ) {
      $clear = sprintf ( "$display_init_special%c", 12 );
      display_position( 1, 1 );

    } else {
      $clear = sprintf ( "%cX", 0xfe );
    }
    print DISPLAY "$clear\n";

  } else {
    system "clear";

  }

}

#
# display_close ( );
#   closes the display
#

sub display_close
{

  if ( $display_type ne "console" )  {
    if ( $display_type eq "crystalfontz" ) {
      $clear = sprintf ( "%c", 12 );
      $off = sprintf ( "%c%c%c", 2, 14, 0 );

    } else {
      $clear = sprintf ( "%cX", 0xfe );
      $off   = sprintf ( "%cF", 0xfe );

    }

    display_clear ( );
    display_position ( 1, 2 );
    display_write ( "   Goodbye..." );
    sleep 2;
    print DISPLAY "$clear$off\n";

    close ( DISPLAY );

  } else  {
    system "stty", 'echo';

  }

}

#
# display_time ( );
#   prints the time in the upper right hand corner
#

sub display_time
{
  ( $sec,$min,$hour,$the_rest ) = localtime ( time );

  $ampm = "a";

  if ( $hour >= 12 )  {
    $hour = $hour - 12 unless ( $hour == 12 );
    $ampm = "p";
  }

  if ( $hour < 1 )  {
    $hour = 12;
  }

  $min  = sprintf ( "%.2d", $min );
  $hour = sprintf ( "%2d", $hour );

  display_position ( 15, 1 );

  if ( $display_type ne "console" ) {
    if ( $display_type eq "crystalfontz" ) {
      $char = sprintf ( "%c%c%c", 27, 91, 67 );

    } else {
      $char = sprintf "%c", 0x00;

    }
    print DISPLAY "$hour$char$min$ampm\n";

  } else {
    print "$hour:$min$ampm\n";

  }

}

#
# display_position ( 5, 1 );
#   positions the cursor at the specified x, y position
#

sub display_position
{
  my ( $x, $y ) = @_;

  if ( $display_type ne "console") {
    if ( $display_type eq "crystalfontz" ) {
      $pos = sprintf ( "%c%c%c", 17, $x - 1, $y - 1 );

    } else {
      $pos = sprintf ( "%cG%c%c", 0xfe, $x, $y );

    }

    print DISPLAY "$pos";

  } else {
    $x --;
    $y --;

    $terminal->Tgoto ('cm', $x, $y, STDOUT);

  }

}

#
# display_check_for_key ( 2 );
#   waits for the specified number of seconds for a keystroke
#

sub display_check_for_key
{

  my ( $timeout ) = @_;
  my $key = $rin = $rout = "";

  if ( $display_type ne "console" && $display_type ne "crystalfontz" )  {

    vec ( $rin, fileno ( DISPLAY ), 1 ) = 1;
    select ( $rout = $rin, undef, undef, $timeout );
    if ( vec ( $rout, fileno ( DISPLAY ), 1 ) )  {
      $key = getc DISPLAY;

    }

  } else {
    system "stty", 'min', '0', '-icanon', 'eol', "\001";

    vec ( $rin, fileno ( STDIN ), 1 ) = 1;
    select ( $rout = $rin, undef, undef, $timeout );
    if ( vec ( $rout, fileno ( STDIN ), 1 ) )  {
      $key = getc ( );

    }

    system "stty", 'icanon', 'eol', '^@';

  }

  return ( $key );

}

#
# display_blink_on ( );
#   turns cursor blinking on
#

sub display_blink_on  {

  $cmd = sprintf "%cS", 0xfe;

  print DISPLAY "$cmd\n" unless ( $display_type eq "console" || $display_type eq "crystalfontz" );

}

#
# display_blink_off  {
#   turns cursor blinking off
#

sub display_blink_off  {

  $cmd = sprintf "%cT", 0xfe;

  print DISPLAY "$cmd\n" unless ( $display_type eq "console" || $display_type eq "crystalfontz" );

}

#
# display_init_bargraph ( );
#  initializes the bargraph system.
#

sub display_init_bargraph {

  if ( $display_type ne "console" )  {
    if ( $display_type eq "crystalfontz" ) {
      my $cmd = sprintf "%c%c%c%c%c%c%c", 18, 0, 255, 0, 14, 0, 2;
      print DISPLAY $cmd;

    } else {
      my $cmd1 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x01,
                        0x00, 0x1f, 0x00, 0x00, 0x00, 0x1f, 0x00, 0x00;
      my $cmd2 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x02,
                        0x00, 0x1f, 0x10, 0x10, 0x10, 0x1f, 0x00, 0x00;
      my $cmd3 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x03,
                          0x00, 0x1f, 0x18, 0x18, 0x18, 0x1f, 0x00, 0x00;
      my $cmd4 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x04,
                        0x00, 0x1f, 0x1c, 0x1c, 0x1c, 0x1f, 0x00, 0x00;
      my $cmd5 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x05,
                        0x00, 0x1f, 0x1e, 0x1e, 0x1e, 0x1f, 0x00, 0x00;
      my $cmd6 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x06,
                        0x00, 0x1f, 0x1f, 0x1f, 0x1f, 0x1f, 0x00, 0x00;
      my $cmd7 = sprintf "%cN%c%c%c%c%c%c%c%c%c", 0xfe, 0x07,
                        0x04, 0x0e, 0x1f, 0x00, 0x1f, 0x0e, 0x04, 0x00;
 
      # $cmd7 builds the little up/down character used in the play mini menu

      print DISPLAY "$cmd1\n$cmd2\n$cmd3\n$cmd4\n$cmd5\n$cmd6\n$cmd7\n";
    }

  }

}

#
# display_bargraph ( $col, $row, $barval, $max_blocks );
#   $max should be a multiple of 5 such as 100. $barval is the number of
#   pixels to graph, so useing all 20 characters, you have a maximum of
#   100 pixels. $row and $col define the block where the graph starts.
#   graphs always run to the right and will not clobber characters after
#   the $max value. (hence the reason i wrote this routine)
#

sub display_bargraph  {

  my ( $col, $row, $barval, $max ) = @_;
  my $max_blocks = int ( $max / 5 );
  my $x = 0;
  my $block = int ( $barval / 5 );
  my $cmd;

  if ( $display_type eq "crystalfontz" ) {
    $col--;
    $row--;
    $max = 1.2 * $max;
    my $max_blocks = int ( $max / 6 );
    $barval = int ( 1.2 * $barval );
    $cmd = sprintf ( "%c%c%c%c%c%c%c", 18, 1, 255, $col, $max_blocks, $barval, $row );

    print DISPLAY $cmd;

  } else {

    display_position ( $col , $row );

    $subbarval = ( $barval - ( $block * 5 ) ) + 1;
    $cmd = sprintf "%c", 0x06;

    while ( $x < $block )  {
      if ( $display_type ne "console" )  {
        print DISPLAY $cmd;

      } else {
        print "*";

      }
      $x ++;

    }

    $subbarval ++;
    $cmd = sprintf "%c", $subbarval;

    if ( $display_type ne "console" ) {
      print DISPLAY $cmd unless ( $barval >= $max );

    }

    $block ++;
    $cmd = sprintf "%c", 0x01;
    $max_blocks -- unless ($display_type ne "console");
    while ( $block < $max_blocks )  {
      if ( $display_type ne "console" ) {
        print DISPLAY $cmd;

      } else {
        print ".";

      }
      $block ++;

    }

  }

}

#
# Display_Scroll Cuts text for the scrolling text feature
# 
 
sub display_scroll {

  my ( $pre_text, $scroll_text, $scroll_start, $scroll_size, $scroll_mode, 
       $scroll_lor, $scroll_percent, $scroll_timer, $scroll_go ) = @_;

  my $scroll_txt_length = length $scroll_text;
  if ( $scroll_txt_length <= $scroll_size ) {
    $return_text = $scroll_text;

  } else {
    my $scroll_dbl_text = $scroll_text . $scroll_text;
    $return_text = substr $scroll_dbl_text, $scroll_start, $scroll_size;

  }

  $return_text = $pre_text . $return_text if ( $pre_text );

  return ( $return_text );

}


#
# Set next Scroll Counter
#
#

sub display_scroll_counter {
  my ( $count_p ) = @_;

  if ( $$count_p[6] ne 0 ) {
    my $count_len=length($$count_p[1]);

    if ($$count_p[4] eq "marquee") {
      if ($$count_p[5] eq "r") {
        $$count_p[2]++;
        $$count_p[2] = 0 if ($$count_p[2] == $count_len);

      } else {
        $$count_p[2]--;
        $$count_p[2] = $count_len if ($$count_p[2] == 0);

      }

    } else {
      if ($$count_p[5] eq "r") {
        $$count_p[2]++;
        $$count_p[5]="l" if ( ( $$count_p[2] + $$count_p[3] ) >= $count_len );

      } else {
        $$count_p[2]--;
        $$count_p[5]="r" if ($$count_p[2] <= 0);

      }

    }

  }

}

#
# Process Scrolling information
#
#

sub display_scroll_code {

  my ( $scroll_p ) = @_;

  if ($$scroll_p[6] ge 0) {
    my $scroll_speed = (-(($$scroll_p[6] / 100) -1) * 40) + 5;
    $$scroll_p[5] = "r" if ($$scroll_p[4] eq "marquee");

    if ($$scroll_p[7] > $scroll_speed) {
      $$scroll_p[8] = 0; 
      $$scroll_p[7] = 0; 

    } else { 
      $$scroll_p[8] = 1; 
      $$scroll_p[7]++;

    }

  } elsif ( $$scroll_p[6] lt 0 ) {
    $scroll_speed = ((($$scroll_p[6] / -100) -1) * 40) - 5;
    $$scroll_p[5] = "l" if ($$scroll_p[4] eq "marquee");
    if ($$scroll_p[7] < $scroll_speed) {
      $$scroll_p[8] = 0;
      $$scroll_p[7] = 0;

    } else { 
      $$scroll_p[8] = 1;
      $$scroll_p[7]--;

    }

  }

}
